const path = require("path");

module.exports = grunt => {
    grunt.initConfig({
        pkg: grunt.file.readJSON("package.json"),
        clean: {
            default: ["dist/"],
        },
        copy: {
            css: {
                files: [{expand: true, cwd: "src/main/css/", src: "**/*.css", dest: "dist/"}],
            },
            html: {
                files: [{expand: true, cwd: "src/main/", src: "**/*.html", dest: "dist/"}],
            },
        },
        focus: {
            dev: {
                include: ["css", "html", "js", "link"],
            },
        },
        replace: {
            dev: {
                src: ["./dist/*.html", "./dist/*.js"],
                replacements: [
                    {
                        from: "%%VERSION_NUMBER%%",
                        to: "<%= pkg.version %>+" + new Date().toISOString().slice(0, 19).replace(/[-:T]/g, ""),
                    }
                ],
                overwrite: true,
            },
            deploy: {
                src: ["./dist/*.html", "./dist/*.js"],
                replacements: [
                    {
                        from: "%%VERSION_NUMBER%%",
                        to: "<%= pkg.version %>",
                    },
                ],
                overwrite: true,
            },
        },
        watch: {
            css: {
                files: ["src/main/**/*.css"],
                tasks: ["copy:css"],
            },
            html: {
                files: ["src/main/**/*.html"],
                tasks: ["copy:html"],
            },
            js: {
                files: ["src/main/**/*.js"],
                tasks: ["webpack:dev", "replace:dev"],
            },
            link: {
                files: ["node_modules/@fwdekker/*/dist/**"],
                tasks: ["webpack:dev", "replace:dev"],
            },
        },
        webpack: {
            options: {
                entry: "./src/main/js/Main.js",
                module: {
                    rules: [
                        {
                            test: /\.js$/,
                            exclude: /node_modules/,
                        },
                        {
                            test: /\.css$/i,
                            use: ["style-loader", "css-loader"],
                        },
                        {
                            test: /\.(woff(2)?|ttf|eot|svg)(\?v=\d+\.\d+\.\d+)?$/,
                            use: [{
                                loader: "file-loader",
                                options: {
                                    name: "[name].[ext]",
                                    outputPath: "./",
                                },
                            }],
                        },
                    ],
                },
                resolve: {
                    extensions: [".js", ".css"],
                },
                output: {
                    filename: "bundle.js",
                    path: path.resolve(__dirname, "dist/"),
                },
            },
            dev: {
                mode: "development",
                devtool: "inline-source-map",
            },
            deploy: {
                mode: "production",
            },
        },
    });

    grunt.loadNpmTasks("grunt-contrib-clean");
    grunt.loadNpmTasks("grunt-contrib-copy");
    grunt.loadNpmTasks("grunt-contrib-watch");
    grunt.loadNpmTasks("grunt-focus");
    grunt.loadNpmTasks("grunt-text-replace");
    grunt.loadNpmTasks("grunt-webpack");

    grunt.registerTask("dev", [
        // Pre
        "clean",
        // Copy files
        "copy:css",
        "copy:html",
        // Compile JS
        "webpack:dev",
        "replace:dev",
    ]);
    grunt.registerTask("dev:server", ["dev", "focus:dev"]);
    grunt.registerTask("deploy", [
        // Pre
        "clean",
        // Copy files
        "copy:css",
        "copy:html",
        // Compile JS
        "webpack:deploy",
        "replace:deploy",
    ]);

    grunt.registerTask("default", ["dev"]);
};
